import React, { useEffect, useState, useCallback } from 'react';
import { sortData, filterData, paginateData } from './helpers/data';
import Pagination from './Pagination';
import PaginationOpts from './PaginationOpts';
import TableHeader from './TableHeader';
import TableBody from './TableBody';
import Filter from './Filter';
import { makeClasses, customJoin, shouldTableUpdate } from './helpers/object';
import Row from 'react-bootstrap/Row';
import Button from 'react-bootstrap/Button';
import Table from 'react-bootstrap/Table';
import Col from 'react-bootstrap/Col';
import ButtonGroup from 'react-bootstrap/ButtonGroup';
import FontAwesome from './components/FontAwesome';
import { useComponentProvider } from './components/TableContext';
/**
 * Datatable lifecycle convenient function.
 * It will be used when we are extending the table.
 **/
export function useDatatableLifecycle({ initialSort, onSort, onFilter, rowsPerPage, rowsPerPageOption, async, tableHeaders, classes, tableBody, labels, Components, onRowClick }) {
    useEffect(() => {
        // If in development, warn if async and onSort/onFilter are both passed.
        if (process.env.NODE_ENV === 'development') {
            if (async !== undefined) {
                // Warn if onSort and/or onFilter is/are also passed down.
                let str = [];
                if (onSort !== undefined) {
                    str.push('[onSort]');
                }
                if (onFilter !== undefined) {
                    str.push('[onFilter]');
                }
                if (str.length > 0) {
                    console.warn(`You are passing [async] props along with ${customJoin(str, ' and ')}. When [async] is enabled, you should not pass onFilter or onSort.`);
                }
                // Warn if all async options are not passed.
                str = [];
                if (async.onFilter === undefined) {
                    str.push('[async.onFilter]');
                }
                if (async.onSort === undefined) {
                    str.push('[async.onSort]');
                }
                if (async.onPaginate === undefined) {
                    str.push('[async.onPaginate]');
                }
                if (str.length > 0) {
                    console.warn(`These async props are missing: ${customJoin(str, ', ', 'and ')}`);
                }
            }
        }
    }, []);
    const [state, setState] = useState(() => {
        const sortObj = initialSort || {};
        let filterable = async !== undefined && async.onFilter !== undefined;
        let defaultSort = {};
        tableHeaders.forEach((header) => {
            if (header.prop === sortObj.prop) {
                if (header.sortable) {
                    defaultSort = {
                        isAscending: true,
                        prop: header.prop
                    };
                }
            }
            if (header.filterable && async === undefined && !filterable) {
                filterable = true;
            }
        });
        // Define initial state.
        return {
            sortedProp: defaultSort,
            filterable,
            rowsPerPage,
            currentPage: 1,
            filterText: ''
        };
    });
    useEffect(() => {
        // Resets the table if the data passed down is different.
        if (tableBody !== undefined) {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { filterText: '', currentPage: 1, rowsPerPage: rowsPerPage })));
        }
    }, [tableBody, rowsPerPage]);
    const onChangeFilter = useCallback((text) => {
        if (async && async.onFilter) {
            async.onFilter(text);
        }
        else {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { filterText: text, currentPage: 1 })));
        }
    }, [async]);
    const onPageNavigate = useCallback((nextPage) => {
        if (async && async.onPaginate) {
            async.onPaginate(nextPage);
        }
        else {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { currentPage: nextPage })));
        }
    }, [async]);
    const onRowsPerPageChange = useCallback((numOfPage) => {
        if (async && async.onRowsPerPageChange) {
            async.onRowsPerPageChange(numOfPage);
        }
        else {
            setState((oldState) => (Object.assign(Object.assign({}, oldState), { rowsPerPage: numOfPage, currentPage: 1 })));
        }
    }, [async]);
    const onSortChange = useCallback((nextProp) => {
        if (async && async.onSort) {
            async.onSort(nextProp);
        }
        else {
            setState((oldState) => {
                const nextSort = oldState.sortedProp;
                if (nextProp !== oldState.sortedProp.prop) {
                    nextSort.prop = nextProp;
                    nextSort.isAscending = true;
                }
                else {
                    nextSort.isAscending = !oldState.sortedProp.isAscending;
                }
                return Object.assign(Object.assign({}, oldState), { sortedProp: nextSort });
            });
        }
    }, [async]);
    let data = tableBody;
    let maxPage;
    if (async === undefined) {
        data = filterData(tableBody, tableHeaders, state.filterText, onFilter);
        data = sortData(data, state.sortedProp, onSort);
        if (state.rowsPerPage) {
            // Pagination needs.
            data = paginateData(data, state.currentPage, state.rowsPerPage);
            maxPage = Math.ceil(tableBody.length / state.rowsPerPage);
        }
    }
    else {
        maxPage = async.maxPage;
    }
    const tableClass = makeClasses(`table-datatable__root`, classes === null || classes === void 0 ? void 0 : classes.table);
    // Default components.
    // If context has keys, then use context. Instead, use Components props.
    const context = useComponentProvider();
    const passedComponents = Object.keys(context).length > 0 ? context : Components;
    let usedComponents = {
        // Global.
        Row,
        Col,
        Button,
        // Table.
        Table,
        TableHead: 'thead',
        TableBody: 'tbody',
        TableRow: 'tr',
        TableCell: 'td',
        // Filter.
        FilterGroup: undefined,
        // Pagination.
        ButtonGroup,
        // Pagination options.
        PaginationOptsGroup: undefined,
        // ICons.
        SortIcon: FontAwesome
    };
    if (passedComponents !== undefined) {
        const keys = Object.keys(passedComponents);
        for (const key of keys) {
            // Replace usedComponent fields with the passedComponents fields.
            usedComponents[key] = passedComponents[key];
        }
    }
    return {
        data,
        tableHeaders,
        onChangeFilter,
        onPageNavigate,
        classes,
        onRowsPerPageChange,
        onSortChange,
        tableClass,
        labels,
        rowsPerPageOption,
        Components: usedComponents,
        onRowClick,
        // States.
        filterable: state.filterable,
        filterText: async ? async.filterText : state.filterText,
        rowsPerPage: async ? async.rowsPerPage : state.rowsPerPage,
        currentPage: async ? async.currentPage : state.currentPage,
        sortedProp: async ? async.sortedProp : state.sortedProp,
        maxPage
    };
}
/** Datatable Component. */
function Datatable(props) {
    var _a;
    const { data, rowsPerPageOption, tableHeaders, onChangeFilter, onPageNavigate, classes, onRowsPerPageChange, onSortChange, tableClass, labels, filterable, filterText, rowsPerPage, currentPage, sortedProp, maxPage, Components, onRowClick } = useDatatableLifecycle(props);
    return (React.createElement(React.Fragment, null,
        React.createElement(Components.Row, { className: makeClasses('controlRow__root', classes === null || classes === void 0 ? void 0 : classes.controlRow) },
            React.createElement(Components.Col, { xs: 12, lg: 4, className: makeClasses('d-flex justify-content-start align-items-end', classes === null || classes === void 0 ? void 0 : classes.filterCol) },
                React.createElement(Filter, { filterable: filterable, classes: classes, placeholder: (_a = labels === null || labels === void 0 ? void 0 : labels.filter) === null || _a === void 0 ? void 0 : _a.filterPlaceholder, onChangeFilter: onChangeFilter, filterText: filterText, CustomFilterGroup: Components.FilterGroup })),
            React.createElement(Components.Col, { xs: 12, sm: 4, lg: 4, className: makeClasses('d-flex justify-content-lg-center justify-content-xs-start align-items-end', classes === null || classes === void 0 ? void 0 : classes.paginationOptsCol) },
                React.createElement(PaginationOpts, { classes: classes, labels: labels, onRowsPerPageChange: onRowsPerPageChange, rowsPerPage: rowsPerPage, rowsPerPageOption: rowsPerPageOption, CustomPaginationOptsGroup: Components.PaginationOptsGroup })),
            React.createElement(Components.Col, { xs: 12, sm: 8, lg: 4, className: makeClasses('d-flex justify-content-end align-items-end', classes === null || classes === void 0 ? void 0 : classes.paginationCol) },
                React.createElement(Pagination, { maxPage: maxPage, classes: classes, rowsPerPage: rowsPerPage, currentPage: currentPage, onPageNavigate: onPageNavigate, labels: labels, components: {
                        Button: Components.Button,
                        ButtonGroup: Components.ButtonGroup
                    } }))),
        React.createElement(Components.Row, null,
            React.createElement(Components.Col, { xs: 12 },
                React.createElement(Table, { className: tableClass },
                    React.createElement(TableHeader, { classes: classes, tableHeaders: tableHeaders, sortedProp: sortedProp, onSortChange: onSortChange, components: {
                            TableHead: Components.TableHead,
                            TableCell: Components.TableCell,
                            TableRow: Components.TableRow
                        } }),
                    React.createElement(TableBody, { classes: classes, tableHeaders: tableHeaders, labels: labels, data: data, components: {
                            TableBody: Components.TableBody,
                            TableCell: Components.TableCell,
                            TableRow: Components.TableRow
                        }, onRowClick: onRowClick }))))));
}
// Only update if rowsPerPage, rowsPerPageOption, and tableBody changes.
export default React.memo(Datatable, shouldTableUpdate);
